package painter;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import painter.palette.Palette;
import painter.palette.PaletteEvent;
import painter.palette.PaletteListener;

@SuppressWarnings("serial")
public class Painter extends JFrame {

	private static final int INITIAL_STROKE_SIZE = 1;

	private static final Color INITIAL_STROKE_COLOR = Color.BLACK;

	private static final Color BACKGROUND_COLOR = Color.WHITE;

	private Canvas canvas = new Canvas(INITIAL_STROKE_COLOR, BACKGROUND_COLOR,
			INITIAL_STROKE_SIZE);

	public Painter() {
		setTitle("Painter");
		setIconImage(fetchImage("/kit.png"));
		setDefaultCloseOperation(EXIT_ON_CLOSE);

		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());

		JSlider slider = createStrokeSizeSlider();
		Palette palette = createStrokeColorPalette();
		JButton clear = createClearButton();

		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(new FlowLayout(FlowLayout.LEADING, 10, 2));
		controlPanel.add(clear);
		controlPanel.add(palette);

		contentPane.add(slider, BorderLayout.LINE_END);
		contentPane.add(canvas, BorderLayout.CENTER);
		contentPane.add(controlPanel, BorderLayout.PAGE_END);
		pack();
		setLocationRelativeTo(null);
	}

	private Image fetchImage(String resourceName) {
		URL url = getClass().getResource(resourceName);
		return Toolkit.getDefaultToolkit().getImage(url);
	}

	private JButton createClearButton() {
		JButton clear = new JButton();
		clear.setIcon(new ImageIcon(fetchImage("/user-trash.png")));
		clear.setToolTipText("Clear");
		clear.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				canvas.clear();
			}

		});
		return clear;
	}

	private Palette createStrokeColorPalette() {
		Palette palette = new Palette();
		palette.addPaletteListener(new PaletteListener() {

			@Override
			public void paletteChanged(PaletteEvent e) {
				Palette p = (Palette) e.getSource();
				canvas.setStrokeColor(p.getSelectedColor());
			}
		});

		return palette;
	}

	private JSlider createStrokeSizeSlider() {
		JSlider slider = new JSlider(JSlider.VERTICAL, 1, 20, 1);
		slider.setMajorTickSpacing(19);
		slider.setMinorTickSpacing(1);
		slider.setPaintTicks(true);
		slider.setPaintLabels(true);
		slider.setSnapToTicks(true);
		slider.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				JSlider s = (JSlider) e.getSource();
				if (!s.getValueIsAdjusting()) {
					canvas.setStrokeSize(s.getValue());
				}
			}
		});

		return slider;
	}

}
