package painter.palette;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.event.EventListenerList;

@SuppressWarnings("serial")
public class Palette extends JPanel {

	private static final int SIZE = 24;

	private Color[] colors = { Color.BLACK, Color.DARK_GRAY, Color.LIGHT_GRAY,
			Color.RED, Color.GREEN, Color.BLUE, Color.CYAN, Color.MAGENTA,
			Color.YELLOW, Color.ORANGE };

	private Color selected = Color.BLACK;

	private EventListenerList listenersList = new EventListenerList();
	private PaletteEvent paletteEvent;

	public Color getSelectedColor() {
		return selected;
	}

	public Palette() {
		setLayout(new FlowLayout());
		ButtonGroup bg = new ButtonGroup();
		for (Color color : colors) {
			JToggleButton b = new JToggleButton();
			b.setSelected(color.equals(selected));
			b.setPreferredSize(new Dimension(SIZE, SIZE));
			b.setAction(new PaletteAction(color));
			b.setToolTipText("RGB: " + color.getRed() + " " + color.getGreen()
					+ " " + color.getBlue());
			bg.add(b);
			add(b);
		}
	}

	public void addPaletteListener(PaletteListener l) {
		listenersList.add(PaletteListener.class, l);
	}

	private void firePaletteChanged() {
		Object[] listeners = listenersList.getListenerList();
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == PaletteListener.class) {
				if (paletteEvent == null) {
					paletteEvent = new PaletteEvent(this);
				}
			}
			((PaletteListener) listeners[i + 1]).paletteChanged(paletteEvent);
		}
	}

	public class PaletteAction extends AbstractAction {

		private Color color;

		public PaletteAction(Color color) {
			super("", new SimpleIcon(color));
			this.color = color;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (!getSelectedColor().equals(color)) {
				selected = color;
				firePaletteChanged();
			}
		}

	}

}